## Copyright (C) 2022 Ioannis Kosmidis

## Distributed as part of the supplementary material for the manuscript
## "Empirical bias-reducing adjustments to estimating functions"
##
## Authors: Ioannis Kosmidis
## Date: 31 March 2021
## Licence: GPL 2 or greater
## NOT A POLISHED PIECE OF PUBLIC-USE SOFTWARE!  Provided "as is".
## NO WARRANTY OF FITNESS FOR ANY PURPOSE!
##
## It is assumed that the user has a Julia and R installation with all
## packages required by the current script and any other script
## included.
##
##
## Code for reproducing the results in Example 6.1


using Random, Distributions, Autologistic, Graphs, FreqTables, Optim, Plots, Printf, RCall, Revise

includet("auto_symmetric_module.jl")

Random.seed!(123)

p = Int64(100)
n = Int64(1000)

nsamples = 100
n_groups = p

beta_nz = 10
nbeta_nz = 10
beta = repeat([10, 10, 5, 5, 0, 0, 0, 0, 0, 0], inner = Int64(p/10))
#2 .* (rand(p - 2 * nbeta_nz) .- 0.5) ./ 10 ## zeros(p - 2 * nbeta_nz)
# beta = vcat(ones(nbeta_nz) * beta_nz, ones(nbeta_nz) * (-beta_nz), last_beta)
var_x = 1 / (2 * n)

normal_d = Normal(0.0, sqrt(var_x))
X = hcat(rand(normal_d, n, p))

# groups = sample(1:n_groups, n, replace = true)
groups = repeat(1:n_groups, inner = Int64(n / n_groups))

elist = [0 0];
for i in unique(groups)
    obs_in_i = findall(groups .== i)
    e = hcat(reshape([ [x, y]  for x= obs_in_i, y = obs_in_i], length(obs_in_i)^2)...)
    e = convert(Matrix, e[:, e[1, :] .!= e[2, :]]')
    global elist = vcat(elist, e)
end
elist = elist[2:size(elist)[1], :];
Adj = zeros(Int64, n, n)
for i in 1:size(elist)[1]
    Adj[elist[i,:1], elist[i,:2]] = 1
end
G = SimpleGraph(Adj)
## Some Y
Y = ones(n)

## subsets (only relevant for RBM)
subsets = [findall(groups .== s) for s in 1:n_groups];

## Data
dat = auto_symmetric.Data(Y, X, G)


# Configure the optimizer
optim_options = Optim.Options(x_tol = 1e-06,
                              f_tol = 1e-06,
                              iterations = 1000,
                              allow_f_increases = true);

lambdas = Vector(-0.2:0.1:0.2)
m = Matrix{Any}(undef, length(lambdas), nsamples)
mp = Matrix{Any}(undef, length(lambdas), nsamples)
rbmp = Matrix{Any}(undef, length(lambdas), nsamples)


for i in 1:length(lambdas)

    lambda = lambdas[i]

    ## truth
    theta = vcat(beta, lambda)


    ## simulate autologistic model
    Random.seed!(123)
    samples = auto_symmetric.simulate(theta, dat, 100)

    Threads.@threads for k in 1:nsamples

        println("λ = $lambda, sample: $k")

        temp_data = auto_symmetric.Data(samples[:, k], X, G)

        start = zeros(length(theta))

        m[i, k] = auto_symmetric.fit(start, temp_data, false, optim_options = optim_options)
        mp[i, k] = auto_symmetric.fit(start, temp_data, false, plugin = true, optim_options = optim_options)

        rbmp[i, k] = auto_symmetric.fit(start, temp_data, true, subsets = subsets, plugin = true, optim_options = optim_options)
    end

end

## Total bias

ran = 1:10
coef_m = Array{Float64}(undef, length(lambdas), p + 1, nsamples)
coef_mp = Array{Float64}(undef, length(lambdas), p + 1, nsamples)
coef_rbmp = Array{Float64}(undef, length(lambdas), p + 1, nsamples)
bias = Matrix{Float64}(undef, length(lambdas), 3)
sd = Matrix{Float64}(undef, length(lambdas), 3)
for i in 1:length(lambdas)
    lambda = lambdas[i]
    theta = vcat(beta, lambda)
    ests_m = map(k -> m[i, k].minimizer, 1:nsamples)
    ests_mp = map(k -> mp[i, k].minimizer, 1:nsamples)
    ests_rbmp = map(k -> rbmp[i, k].minimizer, 1:nsamples)
    ests_m = permutedims(hcat(ests_m...)')
    ests_mp = permutedims(hcat(ests_mp...)')
    ests_rbmp = permutedims(hcat(ests_rbmp...)')
    coef_m[i, :, :] = ests_m
    coef_mp[i, :, :] = ests_mp
    coef_rbmp[i, :, :] = ests_rbmp
    bias[i,:] = [mean(ests_m[ran,:] .- theta[ran]), mean(ests_mp[ran,:] .- theta[ran]), mean(ests_rbmp[ran,:] .- theta[ran])]
    sd[i,:] = sqrt.([var(ests_m), var(ests_mp), var(ests_rbmp)])
end


objects = ["n", "p", "nsamples", "beta", "var_x", "X", "n_groups", "groups",
           "subsets", "lambdas", "coef_m", "coef_mp", "coef_rbmp"]
file_name = "images/highdim_autologistic.rda"
for i in eachindex(objects)
    current_object = objects[i]
    val = eval(Meta.parse(current_object))
    R"""
     assign($current_object, $val)
    """
end
## save and plot
R"""
save(list = $objects, file = $file_name)
"""
