## Copyright (C) 2022 Ioannis Kosmidis

## Distributed as part of the supplementary material for the manuscript
## "Empirical bias-reducing adjustments to estimating functions"
##
## Authors: Ioannis Kosmidis
## Licence: GPL 2 or greater
## NOT A POLISHED PIECE OF PUBLIC-USE SOFTWARE!  Provided "as is".
## NO WARRANTY OF FITNESS FOR ANY PURPOSE!
##
## It is assumed that the user has a Julia and R installation with all
## packages required by the current script and any other script
## included.
##
##
## Code to reproduce the simulation study in Example 5.3

library("parallel")
library("enrichwith")
library("brglm2")
library("detectseparation")
library("optimx")
source("probit_ms_functions.R")

## save path
image_path <- "images/"
## number of base simulations
n_simu <- 1000
## number of cores
n_cores <- 10
## link
link <- make.link("probit")
## number of rows in X matrix
n_obs_vec <- c(100, 150, 200, 250, 300)
## True parameter values
betas <- c(-2, 2, 2, -0.5, -0.5)
## True model
wmod <- paste(which(betas != 0), collapse = "-")


for (n_obs in n_obs_vec) {
    ## Model matrix
    set.seed(123)
    x1 <- rnorm(n_obs)
    x2 <- rbinom(n_obs, 1, 0.25)
    x3 <- rbinom(n_obs, 1, 0.75)
    x4 <- rexp(n_obs)
    X <- cbind(1, x1, x2, x3, x4)
    colnames(X) <- c("Intercept", "X1", "X2", "X3", "X4")
    ## Linear predictors
    etas <- drop(X %*% betas)
    ## Dummy fit to be enriched with scores, information and simulate function
    y <- rbinom(nrow(X), 1, link$linkinv(etas))
    temp_ml <- glm(y ~ -1 + X, family = binomial(link))
    simu <- get_simulate_function(temp_ml)
    simu_data <- simu(betas, nsim = n_simu, seed = 123)
    res <- mclapply(seq.int(n_simu), function(j) {
        out <- try(fit_model(X, simu_data[, j], link))
        if (j %% 100 == 0) cat("sample:", j, "nobs", n_obs, "Done!\n")
        list(coef_ml = out$coef_ml,
             coef_br = out$coef_br,
             coef_tr = out$coef_tr,
             coef_tr_extra = out$coef_tr_extra,
             coef_erbm = out$coef_erbm,
             scores_ml = out$scores_ml,
             scores_tr = out$scores_tr,
             scores_tr_extra = out$scores_tr_extra,
             scores_br = out$scores_br,
             scores_erbm = out$scores_erbm,
             separated = out$separated,
             converged_ml = out$converged_ml,
             converged_br = out$converged_br,
             converged_tr = out$converged_tr,
             converged_tr_extra = out$converged_tr_extra,
             infinite_ml = out$infinite_ml)
    }, mc.cores = n_cores)
    save(list = c("res", "n_obs", "n_simu", "X", "betas", "simu_data"),
         file = file.path(image_path, paste0("bias_", link$name, "_", n_obs, ".rda")))
}
