## Copyright (C) 2022 Ioannis Kosmidis

## Distributed as part of the supplementary material for the manuscript
## "Empirical bias-reducing adjustments to estimating functions"
##
## Authors: Ioannis Kosmidis
## Licence: GPL 2 or greater
## NOT A POLISHED PIECE OF PUBLIC-USE SOFTWARE!  Provided "as is".
## NO WARRANTY OF FITNESS FOR ANY PURPOSE!
##
## It is assumed that the user has a Julia and R installation with all
## packages required by the current script and any other script
## included.
##
##
## Code to reproduce the simulation study in Example 5.5

library("parallel")
library("enrichwith")
library("brglm2")
library("detectseparation")
source("probit_ms_functions.R")

## save path
image_path <- "images/"
## number of base simulations
n_simu <- 10000
## number of cores
n_cores <- 8
## link
link <- make.link("probit")
## number of rows in X matrix
n_obs_vec <- c(75, 150, 300, 600)
## True parameter values
betas <- c(-1, 0, 0, 1, 1) / 2
## True model
wmod <- paste(which(betas != 0), collapse = "-")

for (n_obs in n_obs_vec) {
    ## Model matrix
    set.seed(123)
    x1 <- rnorm(n_obs)
    x2 <- rbinom(n_obs, 1, 0.25)
    x3 <- rbinom(n_obs, 1, 0.75)
    x4 <- rexp(n_obs)
    X <- cbind(1, x1, x2, x3, x4)
    colnames(X) <- c("Intercept", "X1", "X2", "X3", "X4")
    ## Linear predictors
    etas <- drop(X %*% betas)
    ## Dummy fit to be enriched with scores, information and simulate function
    y <- rbinom(nrow(X), 1, link$linkinv(etas))
    temp_ml <- glm(y ~ -1 + X, family = binomial(link))
    simu <- get_simulate_function(temp_ml)
    simu_data <- simu(betas, nsim = n_simu, seed = 123)
    res <- mclapply(seq.int(n_simu), function(j) {
        out <- try(fit_all_models(X, simu_data[, j], link))
        model_names <- rownames(out$models)
        crit_ml <- colnames(out$ic_ml)
        crit_tr <- colnames(out$ic_tr)
        crit_tr_extra <- colnames(out$ic_tr_extra)
        crit_br <- colnames(out$ic_br)
        selected_model_ml <- structure(model_names[apply(out$ic_ml, 2, which.min)],
                                       names = crit_ml)
        selected_model_tr <- structure(model_names[apply(out$ic_tr, 2, which.min)],
                                       names = crit_tr)
        selected_model_tr_extra <- structure(model_names[apply(out$ic_tr_extra, 2, which.min)],
                                             names = crit_tr_extra)
        selected_model_br <- structure(model_names[apply(out$ic_br, 2, which.min)],
                                       names = crit_br)
        if (j %% 100 == 0) cat("sample:", j, "nobs", n_obs, "Done!\n")
        list(model_ml = selected_model_ml,
             model_br = selected_model_br,
             model_tr = selected_model_tr,
             model_tr_extra = selected_model_tr_extra,
             coef_ml = out$coef_ml,
             coef_br = out$coef_br,
             coef_tr = out$coef_tr,
             coef_tr_extra = out$coef_tr_extra,
             separated = out$separated,
             converged_ml = out$converged_ml,
             converged_br = out$converged_br,
             converged_tr = out$converged_tr,
             converged_tr_extra = out$converged_tr_extra,
             infinite_ml = out$infinite_ml)
    }, mc.cores = n_cores)
    save(list = c("res", "n_obs", "n_simu", "X", "betas", "simu_data"),
         file = file.path(image_path, paste0("model_selection_", link$name, "_", n_obs, ".rda")))
}
