## Copyright (C) 2022 Ioannis Kosmidis

## Distributed as part of the supplementary material for the manuscript
## "Empirical bias-reducing adjustments to estimating functions"
##
## Authors: Ioannis Kosmidis
## Licence: GPL 2 or greater
## NOT A POLISHED PIECE OF PUBLIC-USE SOFTWARE!  Provided "as is".
## NO WARRANTY OF FITNESS FOR ANY PURPOSE!
##
## It is assumed that the user has a Julia and R installation with all
## packages required by the current script and any other script
## included.
##
##
## Code to reproduce the results in Figure S8


path <- "."
setting_scripts0 <- c("arxiv-2010.13687-S6.R",
                     "arxiv-2204.07907-SJ4-I.R",
                     "arxiv-2204.07907-SJ4-II.R",
                     "arxiv-2204.07907-SJ4-III.R")

titles <- c("arxiv:2010.13687v2, \u00A76",
            "arxiv:2204.07907v1, \u00A7J.4",
            "arxiv:2204.07907v1, \u00A7J.4",
            "arxiv:2204.07907v1, \u00A7J.4")

library("MASS")
library("dplyr")
library("ggplot2")
library("patchwork")
library("colorspace")

source(file.path(path, "negbin-functions.R"))

res_all <- NULL
for (k in seq_along(setting_scripts0))  {
    source(file.path(path, setting_scripts0[k]))
    img_name <- paste0("negbin-results-", "n", n, "-p", length(betas), "-kappa", kappa, ".rda")
    load(file.path(path, img_name))
    cat(setting_scripts0[k], "\n")
    res_long <- do.call("rbind",  c(res_ml, res_erbm, res_irbm, res_br))
    res_long$parameter <- names(pars0)
    res_long$truth <- pars0
    res_long <- res_long |> mutate(nparameter = case_when(
                                       parameter == "X01" ~ "beta[1]",
                                       parameter == "X02" ~ "beta[2]",
                                       parameter == "X03" ~ "beta[3]",
                                       parameter == "kappa" ~ "kappa",
                                       TRUE ~ paste0("beta[4:", length(betas), "]")))
    res_long$setting <- paste0(titles[k], "\nn = ", n, ", p = ", length(betas))
    res_all <- rbind(res_all, res_long)
}

res_all$method <- factor(res_all$method, levels = c("ML", "iRBM", "eRBM", "AS"), ordered = TRUE)

size <- 1.5

summaries <- res_all |>
    group_by(setting, method, nparameter) |>
    summarize(
        expectation = mean(estimate, na.rm = TRUE),
        bias = mean(estimate - truth, na.rm = TRUE),
        rmse = sqrt(mean((estimate - truth)^2, na.rm = TRUE)),
        sd = sd(estimate, na.rm = TRUE),
        rmseOsd = rmse/sd,
        pu = mean(estimate < truth, na.rm = TRUE),
        nvalid  = sum(!is.na(estimate)),
        prop_valid = nvalid / n(),
        sd = sd(estimate, na.rm = TRUE) / sqrt(nvalid),
        abs_bias = abs(bias)) |>
    data.frame()

## barplots
bias <- ggplot(summaries) +
    geom_col(aes(x = nparameter, y = abs(bias), fill = method), position = "dodge2") +
    geom_hline(aes(yintercept = 0), colour = "black", lty = 2, alpha = 0.5) +
    facet_wrap(~ setting, scales = "free_x", nrow = 1) +
    scale_x_discrete(labels = function(x) parse(text = x)) +
    labs(x = "Parameter", y = "|Bias|") +
    theme_minimal() +
    theme(legend.position = "bottom") +
    scale_fill_grey()

rmse <- ggplot(summaries) +
    geom_col(aes(x = nparameter, y = rmse, fill = method), position = "dodge2") +
    facet_wrap(~ setting, scales = "free_x", nrow = 1) +
    scale_x_discrete(labels = function(x) parse(text = x)) +
    labs(x = "Parameter", y = "RMSE") +
    theme_minimal() +
    theme(legend.position = "bottom") +
    scale_fill_grey()

noxticks <- theme(axis.ticks.x = element_blank(), axis.text.x = element_blank())
nolegend <- theme(legend.position = "none")
nolabels <- theme(strip.text.x = element_blank())

pdf("negbin_bias.pdf", width = 10, height = 5)
(bias + labs(x = NULL) + noxticks + nolegend) /
    (rmse + nolabels + nolegend)
dev.off()
